# Generated by roxygen2: do not edit by hand

S3method(print,analysis_list)
S3method(print,decision_list)
S3method(print,scenario_list)
export(combinePriorParameters)
export(continueRecruitment)
export(createTrial)
export(getAverageNSubjects)
export(getEstimates)
export(getGoDecisions)
export(getGoProbabilities)
export(getMuVar)
export(getPriorParameters)
export(invLogit)
export(loadAnalyses)
export(loadScenarios)
export(logit)
export(negateGoDecisions)
export(performAnalyses)
export(saveAnalyses)
export(saveScenarios)
export(scaleRoundList)
export(setPriorParametersBerry)
export(setPriorParametersExNex)
export(setPriorParametersExNexAdj)
export(setPriorParametersPooled)
export(setPriorParametersStratified)
export(setPriorParametersStratifiedMix)
export(simulateScenarios)
