<!-- badges: start -->
[![Linux](https://github.com/Boehringer-Ingelheim/bhmbasket/actions/workflows/linux.yml/badge.svg?branch=test)](https://github.com/Boehringer-Ingelheim/bhmbasket/actions/workflows/linux.yml)
[![Windows](https://github.com/Boehringer-Ingelheim/bhmbasket/actions/workflows/windows.yml/badge.svg?branch=test)](https://github.com/Boehringer-Ingelheim/bhmbasket/actions/workflows/windows.yml)
[![macOS](https://github.com/Boehringer-Ingelheim/bhmbasket/actions/workflows/macos.yml/badge.svg?branch=test)](https://github.com/Boehringer-Ingelheim/bhmbasket/actions/workflows/macos.yml)
[![Codecov test coverage](https://codecov.io/gh/Boehringer-Ingelheim/bhmbasket/branch/test/graph/badge.svg)](https://app.codecov.io/gh/Boehringer-Ingelheim/bhmbasket?branch=test)
<!-- badges: end -->


# The `bhmbasket` package

Simulate and analyze basket trial designs with binary endpoints using Bayesian hierarchical models.

## Installation

Install the package from CRAN using:

```{r}
install.packages("bhmbasket")
```

The development version can be installed from this repository:

```{r}
# install.packages("remotes")
remotes::install_github("https://github.com/Boehringer-Ingelheim/bhmbasket")
```
